package com.appier.appier_flutter

import android.os.Handler
import android.os.Looper
import android.app.Activity
import android.app.Application
import android.content.Context
import com.quantumgraph.sdk.CompletionHandler;
import com.quantumgraph.sdk.QG
import io.flutter.plugin.common.MethodChannel
import org.json.JSONObject

class AppierFlutter(private val context: Context) {
  private var activity: Activity? = null
  private val handler: Handler = Handler(Looper.getMainLooper())

  fun setActivity(activity: Activity?) {
    this.activity = activity
  }

  fun configure(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val appId = arguments["appId"] as? String
    if (appId == null) {
      result.error("ILLEGAL_ARGUMENT", "configure: appId should not be null", null)
      return
    }

    val application = context.applicationContext as Application
    val version = arguments["version"] as String
    val senderId = arguments["senderId"] as? String
    if (senderId != null) {
      QG.initializeSdkForHybrid(application, "flutter", version, appId, senderId)
    } else {
      QG.initializeSdkForHybrid(application, "flutter", version, appId)
    }
    QG.initializeInApp(application)
    activity?.let {
      QG.setInAppCurrActivity(it)
      result.success(null)
      return
    }
    result.error("ILLEGAL_ARGUMENT", "activity should not be null, please call setActivity() before this configure", null)
  }

  fun renewAppierId(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    QG.getInstance(context).renewAppierId()
    result.success(null)
  }

  fun getAppierId(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val appierId = QG.getInstance(context).getAppierId()
    result.success(appierId)
  }

  fun setUniversalLinkDomains(result: MethodChannel.Result) {
    result.error("NOT_SUPPORTED", "setUniversalLinkDomains}: method is not supported in android", null)
  }

  @Suppress("UNCHECKED_CAST")
  fun logEvent(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val eventName = arguments["eventName"] as String
    val eventInfo = (arguments["parameters"] as? HashMap<String, Any?>)?.let { params -> JSONObject(params) }
    val vts = arguments["vts"] as? Double
    val vtsCurr = arguments["vtsCurr"] as? String
    if (vts == null && vtsCurr != null) {
      result.error("ILLEGAL_ARGUMENT", "logEvent: valueToSum shouldn't be null if the currency is not null", null)
      return
    }
    QG.getInstance(context).logEvent(eventName, eventInfo, vts, vtsCurr)
    result.success(null)
  }

  fun flush(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    QG.getInstance(context).flush()
    result.success(null)
  }

  fun setUserId(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val userId = arguments["userId"] as? String
    if (userId.isNullOrEmpty()) {
      result.error("ILLEGAL_ARGUMENT", "setUserId: userId should not be null or empty", null)
      return
    }

    QG.getInstance(context).setUserId(userId)
    result.success(null)
  }

  fun setName(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val name = arguments["name"] as? String
    if (name.isNullOrEmpty()) {
      result.error("ILLEGAL_ARGUMENT", "setName: name should not be null or empty", null)
      return
    }

    QG.getInstance(context).setName(name)
    result.success(null)
  }

  fun setFirstName(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val firstName = arguments["firstName"] as? String
    if (firstName.isNullOrEmpty()) {
      result.error("ILLEGAL_ARGUMENT", "setFirstName: firstName should not be null or empty", null)
      return
    }

    QG.getInstance(context).setFirstName(firstName)
    result.success(null)
  }

  fun setLastName(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val lastName = arguments["lastName"] as? String
    if (lastName.isNullOrEmpty()) {
      result.error("ILLEGAL_ARGUMENT", "setLastName: lastName should not be null or empty", null)
      return
    }

    QG.getInstance(context).setLastName(lastName)
    result.success(null)
  }

  fun setCity(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val city = arguments["city"] as? String
    if (city.isNullOrEmpty()) {
      result.error("ILLEGAL_ARGUMENT", "setCity: city should not be null or empty", null)
      return
    }

    QG.getInstance(context).setCity(city)
    result.success(null)
  }

  fun setEmail(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val email = arguments["email"] as? String
    if (email.isNullOrEmpty()) {
      result.error("ILLEGAL_ARGUMENT", "setEmail: email should not be null or empty", null)
      return
    }

    QG.getInstance(context).setEmail(email)
    result.success(null)
  }

  fun setPhoneNumber(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val phoneNo = arguments["phoneNo"] as? String
    if (phoneNo.isNullOrEmpty()) {
      result.error("ILLEGAL_ARGUMENT", "setPhoneNumber: phoneNo should not be null or empty", null)
      return
    }

    QG.getInstance(context).setPhoneNumber(phoneNo)
    result.success(null)
  }

  fun setDayOfBirth(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val day = arguments["day"] as? Int
    if (day == null || day <= 0) {
      result.error("ILLEGAL_ARGUMENT", "setDayOfBirth: day should not be null or non-positive", null)
      return
    }

    QG.getInstance(context).setDayOfBirth(day)
    result.success(null)
  }

  fun setMonthOfBirth(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val month = arguments["month"] as? Int
    if (month == null || month <= 0) {
      result.error("ILLEGAL_ARGUMENT", "setMonthOfBirth: month should not be null or non-positive", null)
      return
    }

    QG.getInstance(context).setMonthOfBirth(month)
    result.success(null)
  }

  fun setYearOfBirth(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val year = arguments["year"] as? Int
    if (year == null || year <= 0) {
      result.error("ILLEGAL_ARGUMENT", "setYearOfBirth: year should not be null or non-positive", null)
      return
    }

    QG.getInstance(context).setYearOfBirth(year)
    result.success(null)
  }

  fun setCustomKey(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val key = arguments["key"] as? String
    if (key == null) {
      result.error("ILLEGAL_ARGUMENT", "setCustomKey: key should not be null", null)
      return
    }

    val value = arguments["value"]
    QG.getInstance(context).setCustomUserParameter(key, value)
    result.success(null)
  }

  fun setAttributionWindow(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val seconds = arguments["seconds"] as? Int
    if (seconds == null || seconds <= 0) {
      result.error("ILLEGAL_ARGUMENT", "setAttributionWindow: seconds should not be null or non-positive", null)
      return
    }

    QG.getInstance(context).setAttributionWindow(seconds.toLong())
    result.success(null)
  }

  fun setClickAttributionWindow(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val seconds = arguments["seconds"] as? Int
    if (seconds == null || seconds <= 0) {
      result.error("ILLEGAL_ARGUMENT", "setClickAttributionWindow: seconds should not be null or non-positive", null)
      return
    }

    QG.getInstance(context).setClickAttributionWindow(seconds.toLong())
    result.success(null)
  }

  fun getRecommendation(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val scenarioId = arguments["scenarioId"] as String
    val productId = arguments["productId"] as? String
    val parameters = (arguments["parameters"] as? HashMap<String, Any?>)?.let { params -> JSONObject(params) }
    QG.getInstance(context).getRecommendationWithScenarioId(scenarioId, productId, parameters, CompletionHandler { jsonObject ->
      handler.post({
        if (jsonObject != null) {
          if (jsonObject.length() == 0) {
            result.success(null)
          } else {
            result.success(jsonObject.toString())
          }
        } else {
          result.success(null)
        }
      })
    })
  }

  fun logRecommendationClicked(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val scenarioId = arguments["scenarioId"] as String
    val productId = arguments["productId"] as String
    val recommendationId = arguments["recommendationId"] as String
    val modelId = when (arguments["modelId"]) {
      is Int -> (arguments["modelId"] as Int).toLong()
      is Long -> arguments["modelId"] as Long
      else -> {
        result.error("ILLEGAL_ARGUMENT", "logRecommendationClicked: modelId should be Int or Long", null)
        return
      }
    }
    QG.getInstance(context).logRecommendationClicked(scenarioId, modelId, productId, recommendationId)
    result.success(null)
  }

  fun handleRemoteMessage(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    val messageString = arguments["messageString"] as String
    QG.getInstance(context).handleRemoteMessagePayload(messageString);
    result.success(null)
  }

  fun hideInAppCampaigns(arguments: HashMap<String, Any?>, result: MethodChannel.Result) {
    QG.getInstance(context).hideInAppCampaigns()
    result.success(null)
  }
}
