package com.appier.appier_flutter

import androidx.annotation.NonNull
import com.quantumgraph.sdk.QG
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.FlutterPlugin.FlutterPluginBinding
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding

/**
 * Plugin implementation that uses the new {@code io.flutter.embedding} package.
 *
 * Instantiate this to gracefully handle activity and context changes.
 */
class AppierFlutterPlugin : FlutterPlugin, ActivityAware {
  private var methodCallHandler: MethodCallHandlerImpl? = null
  private var appierFlutter: AppierFlutter? = null
  private var inAppInitialized = false

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPluginBinding) {
    val _appierFlutter = AppierFlutter(flutterPluginBinding.applicationContext)
    appierFlutter = _appierFlutter
    methodCallHandler = MethodCallHandlerImpl(_appierFlutter)
    methodCallHandler?.startListening(flutterPluginBinding.binaryMessenger)
  }
  
  override fun onDetachedFromEngine(binding: FlutterPluginBinding) {
    methodCallHandler?.stopListening()
    methodCallHandler = null
    appierFlutter = null
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    appierFlutter?.setActivity(binding.activity)
    if (!inAppInitialized) {
      QG.initializeInApp(binding.activity.application)
      inAppInitialized = true
    }
  }

  override fun onDetachedFromActivity() {
    appierFlutter?.setActivity(null)
  }

  override fun onDetachedFromActivityForConfigChanges() {
    onDetachedFromActivity()
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    onAttachedToActivity(binding)
  }
}
